@@ -5,7 +5,7 @@ Uses pattern guards
 This is an interpreter of the Unlambda language, written in
 the pure, lazy, functional language Haskell.
 
-Copyright (C) 2001 by rjan Johansen <oerjan@nvg.ntnu.no>
+Copyright (C) 2001 by Ørjan Johansen <oerjan@nvg.ntnu.no>
 Copyright (C) 2006 by Don Stewart - http://www.cse.unsw.edu.au/~dons
 
 This program is free software; you can redistribute it and/or modify
@@ -28,7 +28,9 @@ module Language.Unlambda where
 #if !MIN_VERSION_base(4,6,0)
 import Prelude hiding(catch)
 #endif
+import Control.Applicative
 import Control.Exception (catch, IOException)
+import Control.Monad (liftM, ap)
 
 ------------------------------------------------------------------------
 -- Abstract syntax
@@ -85,6 +87,16 @@ newtype Eval a = Eval ((Maybe Char, Int) -> Cont a -> IO Exp)
 
 type Cont a = (Maybe Char, Int) -> a -> IO Exp
 
+instance Functor Eval where
+
+  fmap = liftM
+
+instance Applicative Eval where
+
+  pure = return
+
+  (<*>) = ap
+
 instance Monad Eval where
 
   (Eval cp1) >>= f = Eval $ \dat1 cont2 ->
