@@ -1,15 +1,23 @@
-{-# LANGUAGE PatternGuards #-}
+{-# LANGUAGE CPP, PatternGuards, ScopedTypeVariables #-}
 
 module Main where
 
 -- A time- and output-limited unlambda
 
+#if !MIN_VERSION_base(4,6,0)
+import Prelude hiding(catch)
+#endif
+import Control.Exception (catch, IOException)
 import System.Posix.Resource
-import Data.Char
-import System.IO
+import Data.Char (toLower)
+import System.IO (hGetChar, hGetLine, stdin, Handle())
 
 import Language.Unlambda
 
+{- WARNING: There is no way this is actually enough sandboxing (mueval has to do
+a heck of a lot more to safely valuate expressions and not all of it
+involves libraries), but since no one seems to be attacking via `unlambda`
+we'll leave this be. -}
 main :: IO Exp
 main = (setResourceLimit ResourceCPUTime $ ResourceLimits (ResourceLimit 5) (ResourceLimit 5)) >> run
 
@@ -24,7 +32,7 @@ run = do
 --
 parse :: Handle -> IO Exp
 parse h = do
-  c <- catch (hGetChar h) (\_ -> error "Parse error at end of file")
+  c <- catch (hGetChar h) (\(_ :: IOException) -> error "Parse error at end of file")
   case toLower c of
     d | d `elem` " \t\n"  -> parse h
     '`' -> do e1 <- parse h
